// file bm_run_0.cc
//
// method to train the boltzmann machine
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: train_cc
//
// arguments:
//      char_1* logfile_a: (output) the log of training iterations
//      char_1* wtfile_a: (output) the files to store the trained weights
//      char_1* infile_a: (input) the files containing training data
//
// return: a logical_1 to indicate success
//
// this method trains the boltzmann machine network using data from a list
// of training files and writes the trained weights to a list of output files
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: train_cc (char_1* logfile_a, 
				     char_1* wtfile_a, char_1* infile_a) {

  // initialize local variables
  //
  int_4 file_count_l = 0;
  float_4* tgt_out_l = new float_4[num_output_neurons_d];
  int_4 symbol_count_l = 0;
  int_4 name_length_l = 0;
  FILE* log_file_l = NULL;
  int_4 i = 0;
  
  // strings and things 
  //
  char_1* dictfile_l = new char_1[MAX_STRING_SIZE];
  char_1* name_spell_l = new char_1[MAX_WORD_SIZE];
  char_1** name_pron_l = new char_1*[MAX_WORD_SIZE];
  for (i = 0; i < MAX_WORD_SIZE; i++) {
    name_pron_l[i] = new char_1[MAX_PHONE_SIZE];
  }
  
  // create and initialize entries in the log file
  //
  log_file_l = open_file_cc (logfile_a, (char_1*)"a");
  fprintf (log_file_l, "Epoch\tTemperature\tAverage error\n");
  fclose (log_file_l);
  
  // initialize the temperature and momentum terms
  //
  num_epochs_d = 0;
  up_momentum_cc ();
  up_temperature_cc ();
    
  // for each pass of input file 
  //
  while (train_status_d == BOLTZ_UNTRAINED) {
  
    // re-initialize file count and reset total error
    //
    file_count_l = 0;
    error_d = 0;
    
    // open the training data file
    //
    FILE* train_file_l = open_file_cc (infile_a, (char_1*)"r");
    
    // read the name of dictionary file from input training file
    //
    while (fscanf (train_file_l, "%s", dictfile_l) != EOF) {
      
      // open the dictionary file and increment file count
      //
      FILE* dict_file_l = open_file_cc (dictfile_l, (char_1*)"r");
      file_count_l++;
      
      //--------------------------------------------------------
      //
      // Operate on each entry in the dictionary. Every word has
      // a time-aligned associated name_pron_l which make up 
      // the input and output respectively
      //
      //--------------------------------------------------------
      while (fscanf (dict_file_l, "%[^\t]", name_spell_l) != EOF) {
	
	// find the length of the name
	//
	name_length_l = strlen ((char*)name_spell_l);
	
	// read in the correct name_pron_l
	//
	read_pron_cc (dict_file_l, name_length_l, name_pron_l);
	
	// load the input shift register
	//
	load_spell_cc (name_spell_l);
	
	// for each letter in the shift register run the
	// training, keep shifting the letters one at a time
	//
	symbol_count_l = 0;
	while (symbol_count_l < name_length_l) {
	  
	  // compute the outputs of the layers
	  //
	  compute_outputs_cc ();
	  
	  // set the expected (target) output for each phoneme
	  //
	  tgt_out_cc (name_pron_l[symbol_count_l], tgt_out_l);
	  
	  // compute updates for the connection weights
	  //
	  compute_updates_cc (tgt_out_l);

	  // compute the net error of the machine
	  //
	  total_error_cc ();
	  
	  // shift the name_spell_l in the register and increment count
	  //
	  shift_word_cc ();
	  symbol_count_l ++;
	}

	// update the connection weights
	//
	up_weights_cc ();
      }
      
      // close files and free memory
      //
      fclose (dict_file_l);
    }
      
    // compute average root mean squared error per training file
    //
    error_d = sqrt (error_d) / file_count_l;

    // increment number of training passes
    //
    num_epochs_d ++;
    
    // make an entry in the log file
    //
    log_file_l = open_file_cc (logfile_a, (char_1*)"a");
    fprintf (log_file_l, "%ld\t%f\t%f\n", num_epochs_d, temperature_d,
	     error_d);
    fclose (log_file_l);
  
    // write the trained network weight matrices to output files
    //
    write_network_cc (wtfile_a);
    
    // update the temperature and momentum terms
    //
    up_temperature_cc ();
    up_momentum_cc ();
    
    // check for when to stop training
    //
    if ((error_d < max_error_d) || (num_epochs_d > num_max_epochs_d - 1)
	|| (temperature_d < min_temp_d)) {
      train_status_d = BOLTZ_TRAINED;
    }

    // close file
    //
    fclose (train_file_l);
  }
  
  // free memory
  //
  delete [] name_pron_l;
  delete [] dictfile_l;
  delete [] name_spell_l;
  delete [] tgt_out_l;
  
  // exit gracefully
  //
  return (ISIP_TRUE);
}


