// file bm_set_0.cc
//
// method to load parameters from file in the boltzmann machine class
//

// isip include files
//
#include "boltz_machine.h"

//-----------------------------------------------------------------------------
// method: set_params_cc
//
// arguments:
//      char_1* wtfile_a: (output) the list of files containing trained weights
//      char_1* func_a: (input) the operation --- train or test
//      char_1* paramfile_a: (input) file containing system parameters
//      int_4 nbest_a: (input) the number of pronunciations
//
// return: a logical_1 to indicate success
//
// this method reads the system parameters from an input file and 
// correspondingly creates the neuron layers in the boltzmann machine, 
// connects them appropriately and initializes the weights with random
// values or seed values as specified
//
//-----------------------------------------------------------------------------

logical_1 Boltz_machine :: set_params_cc (char_1* wtfile_a, char_1* func_a, 
					  char_1* paramfile_a, int_4 nbest_a) {
  
  // set the value of n-best related parameters
  //
  n_best_d = nbest_a;
  num_run_attempts_d *= n_best_d;

  // temporary index variables
  //
  int_4 i = 0;
  int_4 k = 0;
  int_4 ind_l = 0;
  
  // open the parameters file
  //
  FILE* prms_file_l = open_file_cc (paramfile_a, (char_1*)"r");
  
  // strings for file i/o
  //
  char_1* temp_str_l = new char_1[MAX_STRING_SIZE];
  char_1* seed_str_l = new char_1[MAX_STRING_SIZE];

  // read the context used
  //
  fscanf (prms_file_l, "%[^:]", temp_str_l);
  fscanf (prms_file_l, "%[:]", temp_str_l);
  fscanf (prms_file_l, "%s", context_type_d);
 
  // set corresponding flag
  //
  if (strcmp ((char*)context_type_d, "short") == 0) {
    context_d = (int_4) BOLTZ_SHORT;
  }
  else if (strcmp ((char*)context_type_d, "long") == 0) {
    context_d = (int_4) BOLTZ_LONG;
  }
  else if (strcmp ((char*)context_type_d, "both") == 0) {
    context_d = (int_4) BOLTZ_BOTH;
  }
  
  // read the context length and the number of hidden layers and neurons
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)) {

    // read the context length
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%ld", &short_context_length_d);
    num_short_inputs_d = short_context_length_d * num_input_bits_d;
    
    // read the number of hidden layers
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%ld", &num_layers_short_d);
    
    // read the number of neurons in each hidden layer
    //
    num_hidden_short_d = new int_4[num_layers_short_d + 1];
    for (i = 0; i < num_layers_short_d; i++) {
      fscanf (prms_file_l, "%[^:]", temp_str_l);
      fscanf (prms_file_l, "%[:]", temp_str_l);
      fscanf (prms_file_l, "%ld", &ind_l);
      num_hidden_short_d[i] = ind_l;
    }

    // for convenience, assign the number of output neurons as the
    // last element in this list
    //
    num_hidden_short_d[num_layers_short_d] = num_output_neurons_d;
  }
  
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)) {

    // read the context length
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%ld", &long_context_length_d);
    num_long_inputs_d = long_context_length_d * num_input_bits_d;

    // read the number of hidden layers
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%ld", &num_layers_long_d);

    // read the number of neurons in each layer
    //
    num_hidden_long_d = new int_4[num_layers_long_d + 1];
    for (i = 0; i < num_layers_long_d; i++) {
      fscanf (prms_file_l, "%[^:]", temp_str_l);
      fscanf (prms_file_l, "%[:]", temp_str_l);
      fscanf (prms_file_l, "%ld", &ind_l);
      num_hidden_long_d[i] = ind_l;
    }

    // for convenience, assign the number of output neurons as the
    // last element in this list
    //
    num_hidden_long_d[num_layers_long_d] = num_output_neurons_d;
  }
  
  // the list of files containing the trained machine weights
  //
  fscanf (prms_file_l, "%[^:]", temp_str_l);
  fscanf (prms_file_l, "%[:]", temp_str_l);
  fscanf (prms_file_l, "%s", wtfile_a);
  
  // set the operation-related flag
  //
  if (strcmp ((char*)func_a, "mlp") == 0) {
    operation_d = (logical_1)MLP_TEST;
  }
  else if (strcmp ((char*)func_a, "train") == 0) {
    operation_d = (logical_1)BOLTZ_TRAIN;
  
    // read parameters concerning training
    //
    // check if this is retraining
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%s", seed_type_d);
    
    // set appropriate flag for seed type
    //
    if (strcmp ((char*)seed_type_d, "short") == 0) {
      seed_d = (int_4) BOLTZ_SHORT;
    }
    else if (strcmp ((char*)seed_type_d, "long") == 0) {
      seed_d = (int_4) BOLTZ_LONG;
    }
    else if (strcmp ((char*)seed_type_d, "both") == 0) {
      seed_d = (int_4) BOLTZ_BOTH;
    }
    
    // read the seed filename if seed context is not none
    //
    if (seed_d != BOLTZ_NONE) {
      fscanf (prms_file_l, "%[^:]", temp_str_l);
      fscanf (prms_file_l, "%[:]", temp_str_l);
      fscanf (prms_file_l, "%s", seed_str_l);
    }
    
    // read the initial temperature for training
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%f", &init_temp_d);
    
    // read the training schedule coefficient and set it as an exponential
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%f", &exponent_d);
    exponent_d = exp (-exponent_d);
    
    // read the option of weighted training
    //
    fscanf (prms_file_l, "%[^:]", temp_str_l);
    fscanf (prms_file_l, "%[:]", temp_str_l);
    fscanf (prms_file_l, "%s", temp_str_l);
    if (strcmp ((char*)temp_str_l, "no") == 0) {
      scale_update_d = ISIP_FALSE;
    }
    
    // read the number of training iterations allowed, if specified
    //
    if (fscanf (prms_file_l, "%[^:]", temp_str_l) != EOF) {
      fscanf (prms_file_l, "%[:]", temp_str_l);
      fscanf (prms_file_l, "%ld", &num_max_epochs_d);
    }
  }
  
  // close parameter file
  //
  fclose (prms_file_l);

  // build the boltzmann machine network according to these parameters
  //
  output_neuron_d = new Boltz_neuron[num_output_neurons_d];
  error_out_d = new float_4[num_output_neurons_d];

  // if training with seed values initialize the models
  //
  FILE* seed_file_l = NULL;
  if (seed_d != BOLTZ_NONE) {
    
    // open seed file
    //
    seed_file_l = open_file_cc (seed_str_l, (char_1*)"r");
  }
    
  // build the short half or long half depending on context
  //
  if ((context_d == BOLTZ_SHORT) || (context_d == BOLTZ_BOTH)
      || (seed_d == BOLTZ_SHORT) || (seed_d == BOLTZ_BOTH)) {

    // create the input buffer
    //
    input_short_d = new Input_buffer (short_context_length_d);

    // create the hidden layers and associated entities
    //
    hidden_short_d = new Boltz_neuron*[num_layers_short_d];
    error_hid_short_d = new float_4*[num_layers_short_d];
    wt_hid_out_short_d = new float_4**[num_layers_short_d];
    up_hid_out_short_d = new float_4**[num_layers_short_d];

    // create the hidden layer neurons and error term arrays
    //
    for (k = 0; k < num_layers_short_d; k++) {
      hidden_short_d[k] = new Boltz_neuron[num_hidden_short_d[k]];
      error_hid_short_d[k] = new float_4[num_hidden_short_d[k]];
    }
    
    // create the connection matrices
    //
    wt_in_hid_short_d = new float_4*[num_short_inputs_d + 1];
    up_in_hid_short_d = new float_4*[num_short_inputs_d + 1];
    for (i = 0; i < num_short_inputs_d + 1; i++) {
      wt_in_hid_short_d[i] = new float_4[num_hidden_short_d[0]];
      up_in_hid_short_d[i] = new float_4[num_hidden_short_d[0]];
      memset (up_in_hid_short_d[i], 0, num_hidden_short_d[0] *
	      sizeof (float_4));
    }
    
    for (k = 0; k < num_layers_short_d; k++) {
      wt_hid_out_short_d[k] = new float_4*[num_hidden_short_d[k] + 1];
      up_hid_out_short_d[k] = new float_4*[num_hidden_short_d[k] + 1];
      for (i = 0; i < num_hidden_short_d[k] + 1; i++) {
	wt_hid_out_short_d[k][i] = new float_4[num_hidden_short_d[k+1]];
	up_hid_out_short_d[k][i] = new float_4[num_hidden_short_d[k+1]];
	memset (up_hid_out_short_d[k][i], 0, num_hidden_short_d[k+1] *
		sizeof (float_4));
      }
    }
    
    // initialize weights with seed values
    //
    if ((seed_d == BOLTZ_SHORT) || (seed_d == BOLTZ_BOTH)) {
      
      // read in seed files
      //
      fscanf (seed_file_l, "%s", temp_str_l);
      init_weights_cc (num_short_inputs_d + 1, num_hidden_short_d[0], 
		       wt_in_hid_short_d, temp_str_l);

      for (k = 0; k < num_layers_short_d; k++) {
	fscanf (seed_file_l, "%s", temp_str_l);
	init_weights_cc (num_hidden_short_d[k] + 1, num_hidden_short_d[k+1],
			 wt_hid_out_short_d[k], temp_str_l);
      }
    }
    
    // or random initialization if training from scratch
    //
    else if (operation_d == BOLTZ_TRAIN) {
      init_weights_cc (num_short_inputs_d + 1, num_hidden_short_d[0],
		       wt_in_hid_short_d, NULL);
      
      for (k = 0; k < num_layers_short_d; k++) {
	init_weights_cc (num_hidden_short_d[k] + 1, num_hidden_short_d[k+1],
			 wt_hid_out_short_d[k], NULL);
      }
    }
  }
  
  if ((context_d == BOLTZ_LONG) || (context_d == BOLTZ_BOTH)
      || (seed_d == BOLTZ_LONG) || (seed_d == BOLTZ_BOTH)) {

    // create the input buffer
    //
    input_long_d = new Input_buffer (long_context_length_d);

    // create the hidden layers and associated entities
    //
    hidden_long_d = new Boltz_neuron*[num_layers_long_d];
    error_hid_long_d = new float_4*[num_layers_long_d];
    wt_hid_out_long_d = new float_4**[num_layers_long_d];
    up_hid_out_long_d = new float_4**[num_layers_long_d];
    
    // create the hidden layer neurons and error term arrays
    //
    for (k = 0; k < num_layers_long_d; k++) {
      hidden_long_d[k] = new Boltz_neuron[num_hidden_long_d[k]];
      error_hid_long_d[k] = new float_4[num_hidden_long_d[k]];
    }
    
    // create the connection matrices
    //
    wt_in_hid_long_d = new float_4*[num_long_inputs_d + 1];
    up_in_hid_long_d = new float_4*[num_long_inputs_d + 1];
    for (i = 0; i < num_long_inputs_d + 1; i++) {
      wt_in_hid_long_d[i] = new float_4[num_hidden_long_d[0]];
      up_in_hid_long_d[i] = new float_4[num_hidden_long_d[0]];
      memset (up_in_hid_long_d[i], 0, num_hidden_long_d[0] * sizeof (float_4));
    }
    
    for (k = 0; k < num_layers_long_d; k++) {
      wt_hid_out_long_d[k] = new float_4*[num_hidden_long_d[k] + 1];
      up_hid_out_long_d[k] = new float_4*[num_hidden_long_d[k] + 1];
      for (i = 0; i < num_hidden_long_d[k] + 1; i++) {
	wt_hid_out_long_d[k][i] = new float_4[num_hidden_long_d[k+1]];
	up_hid_out_long_d[k][i] = new float_4[num_hidden_long_d[k+1]];
	memset (up_hid_out_long_d[k][i], 0, num_hidden_long_d[k+1] *
		sizeof (float_4));
      }
    }
    
    // initialize weights with seed values
    //
    if ((seed_d == BOLTZ_LONG) || (seed_d == BOLTZ_BOTH)) {
      
      // read in seed files
      //
      fscanf (seed_file_l, "%s", temp_str_l);
      init_weights_cc (num_long_inputs_d + 1, num_hidden_long_d[0], 
		       wt_in_hid_long_d, temp_str_l);

      for (k = 0; k < num_layers_long_d; k++) {
	fscanf (seed_file_l, "%s", temp_str_l);
	init_weights_cc (num_hidden_long_d[k] + 1, num_hidden_long_d[k+1],
			 wt_hid_out_long_d[k], temp_str_l);
      }
    }
    
    // or random initialization if training from scratch
    //
    else if (operation_d == BOLTZ_TRAIN) {
      init_weights_cc (num_long_inputs_d + 1, num_hidden_long_d[0],
		       wt_in_hid_long_d, NULL);

      for (k = 0; k < num_layers_long_d; k++) {
	init_weights_cc (num_hidden_long_d[k] + 1, num_hidden_long_d[k+1],
			 wt_hid_out_long_d[k], NULL);
      }
    }
  }

  // close seed file
  //
  if (seed_d != BOLTZ_NONE) {
    fclose (seed_file_l);
  }

  // free memory
  //
  delete [] temp_str_l;
  delete [] seed_str_l;

  // exit gracefully
  //
  return (ISIP_TRUE);
}
