// file: boltz_random.cc
//
// random number generator

// isip include files
//
#include "boltz_random.h"

//-----------------------------------------------------------------------------
// method: boltz_random_cc
//
// arguments: 
//     float_4 low_a: (input) the lower bound on random values
//     float_4 high_a: (input) the higher bound on random values
//
// return: a float_4 random number within the specified range
//
// this function uses Knuth's subtractive algorithm to generate
// uniform random numbers in (0, 1) and then scales them to the
// desired limits
//
//-----------------------------------------------------------------------------
float_4 boltz_random_cc (float_4 low_a, float_4 high_a) {
  
  // define initialization variables
  //
  static int_4 rand_init_l = BOLTZ_RAND_INIT;
  static int_4 temp_init_l = 0;
  
  // define local variables
  //
  static int_4 *m_array_l  = new int_4 [BOLTZ_RAND_ARRAY];
  static int_4 next_i_l    = 0;
  static int_4 next_ip_l   = 0;
  
  // define temporary variables
  //
  int_4 mj_l = 0;
  int_4 mk_l = 0;
  int_4 ii_l = 0;
  int_4 i = 0;
  int_4 k = 0;
  
  // initialize generator
  //
  if ((rand_init_l < 0) || (temp_init_l == 0)) {
    temp_init_l = 1;
    mj_l = BOLTZ_RAND_SEED - (rand_init_l < 0 ? -rand_init_l : rand_init_l);
    mj_l %= BOLTZ_RAND_BIG;
    m_array_l[BOLTZ_RAND_ARRAY - 1] = mj_l;
    mk_l = 1;
    
    // now initialize rest of array
    //
    ii_l = 0;
    for (i = 0; i < BOLTZ_RAND_ARRAY - 1; i++) {
      ii_l = (21 * i) % (BOLTZ_RAND_ARRAY - 1);
      m_array_l[ii_l] = mk_l;
      mk_l = mj_l - mk_l;
      if (mk_l < 0) {
	mk_l += BOLTZ_RAND_BIG;
      }
      mj_l = m_array_l[ii_l];
    }
    
    // randomize these numbers
    //
    for (k = 0; k < 5; k ++) {
      for (i = 0; i < BOLTZ_RAND_ARRAY; i++) {
	m_array_l[i] -= m_array_l[1 + (i + BOLTZ_RAND_CONST - 1) % 
				 (BOLTZ_RAND_ARRAY - 1)];
	if (m_array_l[i] < 0) {
	  m_array_l[i] += BOLTZ_RAND_BIG;
	}
      }
    }
    
    // set values for next round
    //
    next_i_l     = 0;
    next_ip_l    = BOLTZ_RAND_CONST;
    rand_init_l  = 1;
  }

  // the actual generator --- this is what is called every time except the
  // first (initialization) time
  //
  if (++next_i_l == BOLTZ_RAND_ARRAY) {
    next_i_l = 1;
  }
  if (++next_ip_l == BOLTZ_RAND_ARRAY) {
    next_ip_l = 1;
  }
  mj_l = m_array_l[next_i_l] - m_array_l[next_ip_l];
  if (mj_l < 0) {
    mj_l += BOLTZ_RAND_BIG;
  }
  m_array_l[next_i_l] = mj_l;
  
  // set the limits as desired
  //
  float_4 float_rand_l = (high_a - low_a) * mj_l * BOLTZ_RAND_SCALE + low_a;
  
  // return the suitably scaled random number
  //
  return (float_rand_l);
}
