// file: tree_train_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree.h"
#include "tree_constants.h"

#include <integral_constants.h>

// method: train_cc
//
// arguments:
//  Tree* parent_a: (output) parent of current node
//  char_1** attrs_a: (input) contexts
//  char_1** classes_a: (input) classes of contexts
//  int_4 count_a: (input) indicating number of data
//  int_4 context_a: (input) indicating the context of data
//
// return:
//  a logical_1 indicating status
//
//
logical_1 Tree::train_cc(Tree* parent_a, char_1** attrs_a, char_1** classes_a,
			 int_4 count_a, int_4 context_a) {

  // check parameters
  //
  if (count_a < 1) {
    error_handler_cc((char_1*)"train_cc",
		     (char_1*)"invalid parameter: count_a");
    return ISIP_FALSE;
  }

  if (context_a < 1) {
    error_handler_cc((char_1*)"train_cc",
		     (char_1*)"invalid parameter: context_a");
    return ISIP_FALSE;
  }
    
  if (attrs_a == (char_1**)NULL) {
        error_handler_cc((char_1*)"train_cc",
			 (char_1*)"invalid parameter: attrs_a");
    return ISIP_FALSE;
  }

  for (int_4 j=0;j<count_a;j++) {
    if (attrs_a[j] == (char_1*)NULL) {
      error_handler_cc((char_1*)"train_cc",
		       (char_1*)"invalid parameter: attrs_a");
      return ISIP_FALSE;
    }
    if (classes_a[j] == (char_1*)NULL) {
      error_handler_cc((char_1*)"train_cc",
		       (char_1*)"invalid parameter: classes_a");
      return ISIP_FALSE;
    }
  }

  if (classes_a == (char_1**)NULL) {
    error_handler_cc((char_1*)"train_cc",
		     (char_1*)"invalid parameter: classes_a");
    return ISIP_FALSE;
  }

  int_4 num_left;
  int_4 num_right;

  // set the internal parent node pointer
  //
  parent_d = parent_a;

  logical_1 do_split = ISIP_FALSE;	// should we split?

  // formulate a question
  //
  if(formulate_question_cc(do_split, attrs_a, count_a, context_a)
     != ISIP_TRUE) {
    error_handler_cc((char_1*)"train_cc",
		     (char_1*)"error in formulating the question");
    return ISIP_FALSE;
  }
  
  if (do_split == ISIP_TRUE) {

    // train the left, or yes half of the tree
    //
    num_left = train_left_cc(attrs_a, classes_a, count_a, context_a);
    
    // train the right, or no half of the tree
    //
    num_right = train_right_cc(attrs_a, classes_a, count_a, context_a);
    
    // sanity check
    //
    if (debug_level_d > ISIP_DEBUG_BRIEF) {
      if ((num_left + num_right) != count_a) {
	error_handler_cc((char_1*)"train_cc",
			 (char_1*)"left count and right count do not match total");
	return ISIP_FALSE;
      }
    }
    
    left_prob_d = (float_8)num_left/(float_8)count_a;
    right_prob_d = (float_8)num_right/(float_8)count_a;
  }
  else {

    // no more splitting is possible, all data strings are identical
    //
    if (class_d != (char_1**)NULL) {
      error_handler_cc((char_1*)"train_cc",
		       (char_1*)"class_d not NULL");
      return ISIP_FALSE;
    }

    // we are at leaf node; now give some space to store the
    // classes 
    //
    class_d = new char_1*[TREE_MAX_NUM_CLASSES];
    class_count_d = new int_4[TREE_MAX_NUM_CLASSES];

    for (int_4 i=0; i<TREE_MAX_NUM_CLASSES; i++) {
      class_d[i] = (char_1*)NULL;
      class_count_d[i] = (int_4)0;
    }

    // make sure we are not out of memory
    //
    if ((class_d == (char_1**)NULL) || (class_count_d == (int_4*)NULL)) {
      error_handler_cc((char_1*)"train_cc",
		       (char_1*)"out of memory");
      return ISIP_FALSE;
    }
    
    // check if class exists; if so, increment counter; if not, add to
    // list
    //
    logical_1 found = ISIP_FALSE;
    int_4 num_classes = 0;
    
    for (int_4 k=0; k<count_a; k++) {

      found = ISIP_FALSE;
      
      // see if the class is already in the list
      //
      for (int_4 l=0; l<num_classes; l++) {
	if (strcmp((char*)class_d[l], (char*)classes_a[k]) == 0) {
	  class_count_d[l]++;
	  found = ISIP_TRUE;
	  break;
	}
      }

      // new class, allocate space, copy, set class_count_d, and
      // increment the current num_classes variable
      //
      if (found != ISIP_TRUE) {
	class_d[num_classes] = new char_1[strlen((char*)classes_a[k])+1];
	if (class_d[num_classes] == (char_1*)NULL) {
	  error_handler_cc((char_1*)"train_cc",
			   (char_1*)"out of memory");
	  return ISIP_FALSE;
	}
	class_count_d[num_classes] = 1;
	strcpy((char*)class_d[num_classes], (char*)classes_a[k]);
	num_classes++;
      }
    }    
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}




