// file: boltz_characters.h
//
// declaration of the input character symbols and output phoneme symbols
// used in the boltzmann machine
//

// make sure definitions are only made once
//
#ifndef __ISIP_BOLTZ_CHARACTERS
#define __ISIP_BOLTZ_CHARACTERS

// isip include files
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#include "boltz_constants.h"
#endif

//-----------------------------------------------------------------------------
//
// general constants
//
//-----------------------------------------------------------------------------

// the total number of phonemes and alphabet characters
//
#define NUM_PHONEMES                 46
#define NUM_CHARS                    30

// define sizes for bits per input and output symbol of the machine
//
#define BITS_PER_IN_CHAR             5
#define BITS_PER_OUT_PHONE           6

//-----------------------------------------------------------------------------
//
// static bit vectors representing characters
//
//-----------------------------------------------------------------------------

// define the set of input characters
//
static float_4 input_char_d[NUM_CHARS] = {
  ' ', '-', '\'', '.',
  'a', 'b', 'c', 'd',
  'e', 'f', 'g', 'h',
  'i', 'j', 'k', 'l',
  'm', 'n', 'o', 'p',
  'q', 'r', 's', 't',
  'u', 'v', 'w', 'x',
  'y', 'z'
};

// define the bit vectors corresponding to different input characters
//
static float_4 char_vector_d[NUM_CHARS][BITS_PER_IN_CHAR] = {
  
  // the bits are in the reverse order as they go from
  // LSB to MSB in the arrays
  //
  {0, 0, 0, 0, 0},      //  0 00000 ' '
  {1, 0, 0, 0, 0},      //  1 00001 '-'
  {0, 1, 0, 0, 0},      //  2 00010 '\''
  {1, 1, 0, 0, 0},      //  3 00011 '.'
  {0, 0, 1, 0, 0},      //  4 00100 'a'
  {1, 0, 1, 0, 0},      //  5 00101 'b'
  {0, 1, 1, 0, 0},      //  6 00110 'c'
  {1, 1, 1, 0, 0},      //  7 00111 'd'
  {0, 0, 0, 1, 0},      //  8 01000 'e'
  {1, 0, 0, 1, 0},      //  9 01001 'f'
  {0, 1, 0, 1, 0},      // 10 01010 'g'
  {1, 1, 0, 1, 0},      // 11 01011 'h'
  {0, 0, 1, 1, 0},      // 12 01100 'i'
  {1, 0, 1, 1, 0},      // 13 01101 'j'
  {0, 1, 1, 1, 0},      // 14 01110 'k'
  {1, 1, 1, 1, 0},      // 15 01111 'l'
  {0, 0, 0, 0, 1},      // 16 10000 'm'
  {1, 0, 0, 0, 1},      // 17 10001 'n'
  {0, 1, 0, 0, 1},      // 18 10010 'o'
  {1, 1, 0, 0, 1},      // 19 10011 'p'
  {0, 0, 1, 0, 1},      // 20 10100 'q'
  {1, 0, 1, 0, 1},      // 21 10101 'r'
  {0, 1, 1, 0, 1},      // 22 10110 's'
  {1, 1, 1, 0, 1},      // 23 10111 't'
  {0, 0, 0, 1, 1},      // 24 11000 'u'
  {1, 0, 0, 1, 1},      // 25 11001 'v'
  {0, 1, 0, 1, 1},      // 26 11010 'w'
  {1, 1, 0, 1, 1},      // 27 11011 'x'
  {0, 0, 1, 1, 1},      // 28 11100 'y'
  {1, 0, 1, 1, 1}       // 29 11101 'z'
};

// define the set of output phonemes
//
static char_1 output_phoneme_d[NUM_PHONEMES][MAX_PHONE_SIZE] = {
  {"_"}, {"i:"}, {"I"}, {"E"},
  {"@"}, {"A"}, {"^"}, {">"},
  {"&"}, {"U"}, {"u"}, {"aI"},
  {"ei"}, {">i"}, {"aU"}, {"oU"},
  {"iU"}, {"3r"}, {"&r"}, {">r"},
  {"p"}, {"b"}, {"t"}, {"d"},
  {"k"}, {"g"}, {"h"}, {"v"},
  {"D"}, {"T"}, {"s"}, {"z"},
  {"S"}, {"Z"}, {"f"}, {"m"},
  {"n"}, {"N"}, {"dZ"}, {"tS"},
  {"l"}, {"9r"}, {"j"}, {"w"},
  {"ks"}, {"&k"}
};

// define the bit vectors corresponding to each phone
//
static float_4 phone_vector_d[NUM_PHONEMES][BITS_PER_OUT_PHONE] = {
  
  // the bits appear in the reverse order as
  // they are defined LSB to MSB
  //
  {0, 0, 0, 0, 0, 0},    //  0 000000 "_"
  {1, 0, 0, 0, 0, 0},    //  1 000001 "i:"
  {0, 1, 0, 0, 0, 0},    //  2 000010 "I"
  {1, 1, 0, 0, 0, 0},    //  3 000011 "E"
  {0, 0, 1, 0, 0, 0},    //  4 000100 "@"
  {1, 0, 1, 0, 0, 0},    //  5 000101 "A"
  {0, 1, 1, 0, 0, 0},    //  6 000110 "^"
  {1, 1, 1, 0, 0, 0},    //  7 000111 ">"
  {0, 0, 0, 1, 0, 0},    //  8 001000 "&"
  {1, 0, 0, 1, 0, 0},    //  9 001001 "U"
  {0, 1, 0, 1, 0, 0},    // 10 001010 "u"
  {1, 1, 0, 1, 0, 0},    // 11 001011 "aI"
  {0, 0, 1, 1, 0, 0},    // 12 001100 "ei"
  {1, 0, 1, 1, 0, 0},    // 13 001101 ">i"
  {0, 1, 1, 1, 0, 0},    // 14 001110 "aU"
  {1, 1, 1, 1, 0, 0},    // 15 001111 "oU"
  {0, 0, 0, 0, 1, 0},    // 16 010000 "iU"
  {1, 0, 0, 0, 1, 0},    // 17 010001 "3r"
  {0, 1, 0, 0, 1, 0},    // 18 010010 "&r"
  {1, 1, 0, 0, 1, 0},    // 19 010011 ">r"
  {0, 0, 1, 0, 1, 0},    // 20 010100 "p"
  {1, 0, 1, 0, 1, 0},    // 21 010101 "b"
  {0, 1, 1, 0, 1, 0},    // 22 010110 "t"
  {1, 1, 1, 0, 1, 0},    // 23 010111 "d"
  {0, 0, 0, 1, 1, 0},    // 24 011000 "k"
  {1, 0, 0, 1, 1, 0},    // 25 011001 "g"
  {0, 1, 0, 1, 1, 0},    // 26 011010 "h"
  {1, 1, 0, 1, 1, 0},    // 27 011011 "v"
  {0, 0, 1, 1, 1, 0},    // 28 011100 "D"
  {1, 0, 1, 1, 1, 0},    // 29 011101 "T"
  {0, 1, 1, 1, 1, 0},    // 30 011110 "s"
  {1, 1, 1, 1, 1, 0},    // 31 011111 "z"
  {0, 0, 0, 0, 0, 1},    // 32 100000 "S"
  {1, 0, 0, 0, 0, 1},    // 33 100001 "Z"
  {0, 1, 0, 0, 0, 1},    // 34 100010 "f"
  {1, 1, 0, 0, 0, 1},    // 35 100011 "m"
  {0, 0, 1, 0, 0, 1},    // 36 100100 "n"
  {1, 0, 1, 0, 0, 1},    // 37 100101 "N"
  {0, 1, 1, 0, 0, 1},    // 38 100110 "dZ"
  {1, 1, 1, 0, 0, 1},    // 39 100111 "tS"
  {0, 0, 0, 1, 0, 1},    // 40 101000 "l"
  {1, 0, 0, 1, 0, 1},    // 41 101001 "9r"
  {0, 1, 0, 1, 0, 1},    // 42 101010 "j"
  {1, 1, 0, 1, 0, 1},    // 43 101011 "w"
  {0, 0, 1, 1, 0, 1},    // 44 101100 "ks"
  {1, 0, 1, 1, 0, 1}     // 45 101101 "&k"
};

static int_4 two_power_d[BITS_PER_OUT_PHONE] = {
  1, 2, 4, 8, 16, 32
};

// end of file
//
#endif
