// file: Boltz_machine.h
//
 
// make sure definitions are only made once
//
#ifndef __ISIP_BOLTZ_MACHINE
#define __ISIP_BOLTZ_MACHINE

// isip include files
//
#ifndef __ISIP_BOLTZ_CONSTANTS
#include <boltz_constants.h>
#endif

#ifndef __ISIP_BOLTZ_RANDOM
#include <boltz_random.h>
#endif

#ifndef __ISIP_BOLTZ_NEURON
#include <boltz_neuron.h>
#endif

#ifndef __ISIP_INPUT_BUFFER
#include <input_buffer.h>
#endif

#ifndef __ISIP_OUTPUT_PRON
#include <output_pron.h>
#endif

// forward declaration of classes
//
class Input_buffer;
class Boltz_neuron;
class Output_pron;

// Boltz_machine: the class that defines the boltzmann machine network
// consisting of an input layer, a hidden layer and an output layer
// this is a feed-forward back-propagation type stochastic network
//
class Boltz_machine {
  
  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:
  
  // boltzmann machine data
  //
  // miscellaneous parameter data
  //
  int_4 n_best_d;                  // n best pronunciations
  int_4 num_run_attempts_d;        // number of tries to get n-best prons
  int_4 num_input_bits_d;          // number of output bits per input pin
  int_4 num_output_bits_d;         // number of output bits per output phone
  logical_1 operation_d;           // the network operation - train or test
  logical_1 scale_update_d;        // flag to indicate weighted updating
  float_4 max_rand_limit_d;        // limit on random initial weights

  // context parameters
  //
  char_1* context_type_d;          // context type used
  char_1* seed_type_d;             // seed context type used for training
  int_4 context_d;                 // flag to indicate context
  int_4 seed_d;                    // flag to indicate seed context
  int_4 short_context_length_d;    // length of short context
  int_4 long_context_length_d;     // length of long context

  // number of connections
  //
  int_4 num_layers_short_d;        // number of short-context hidden layers
  int_4 num_layers_long_d;         // number of long-context hidden layers
  int_4 num_short_inputs_d;        // neurons in the short-context input
  int_4 num_long_inputs_d;         // neurons in the long-context input
  int_4* num_hidden_short_d;       // number of neurons in the hidden layers
  int_4* num_hidden_long_d;        // number of neurons in the hidden layers
  int_4 num_output_neurons_d;      // number of neurons in the output layer

  // the neuron layers
  //
  Input_buffer* input_short_d;     // input layer of neurons (short context)
  Input_buffer* input_long_d;      // input layer of neurons (long context)
  Boltz_neuron** hidden_short_d;   // hidden layers of neurons (short context)
  Boltz_neuron** hidden_long_d;    // hidden layers of neurons (long context)
  Boltz_neuron* output_neuron_d;   // output layer of neurons
  
  // the connection weight matrices
  //
  float_4** wt_in_hid_short_d;     // input -> hidden (short context)
  float_4*** wt_hid_out_short_d;   // hidden -> output (short context)
  float_4** wt_in_hid_long_d;      // input -> hidden (long context)
  float_4*** wt_hid_out_long_d;    // hidden -> output (long context)
  
  // the update weight matrices
  //
  float_4** up_in_hid_short_d;     // input -> hidden (short context)
  float_4*** up_hid_out_short_d;   // hidden -> output (short context)
  float_4** up_in_hid_long_d;      // input -> hidden (long context)
  float_4*** up_hid_out_long_d;    // hidden -> output (long context)
  
  // the error terms
  //
  float_4** error_hid_short_d;     // error in hid layer output (short context)
  float_4** error_hid_long_d;      // error in hid layer output (long context)
  float_4* error_out_d;            // error in output layer output

  // network parameters related to training
  //
  int_4 num_epochs_d;         // number of training iterations
  int_4 num_max_epochs_d;     // max number of training iterations
  float_4 error_d;            // total error during training
  float_4 max_error_d;        // maximum allowable error in learning  
  float_4 init_temp_d;        // initial temperature for learning  
  float_4 temperature_d;      // temperature for simulated annealing
  float_4 min_temp_d;         // minimum allowed temperature
  float_4 momentum_d;         // momentum for updating weights
  float_4 final_momentum_d;   // final momentum for learning
  float_4 exponent_d;         // exponential rate for change in temp
  float_4 learning_rate_d;    // learning rate coefficient
  logical_1 train_status_d;   // flag for training status
  
  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:
  
  // constructors and destructors
  //
  ~Boltz_machine ();
  Boltz_machine ();
  
  // set system parameters
  //
  logical_1 set_params_cc (char_1* wtfile_a, char_1* func_a, 
			   char_1* paramfile_a, int_4 nbest_a);

  // input/output methods
  //
  logical_1 get_likelihood_cc (float_4 &lhood_a);  // compute output likelihood
  logical_1 get_phoneme_cc (char_1* pron_a);       // get the output phoneme
  
  // training and application
  //
  logical_1 train_cc (char_1* logfile_a, char_1* wtfile_a, char_1* infile_a);
  logical_1 test_cc (char_1* outfile_a, char_1* wtfile_a, char_1* infile_a);

  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
  
  // file i/o methods
  //
  FILE* open_file_cc (char_1* name_a, char_1* mode_a);
  logical_1 read_network_cc (char_1* infile_a);
  logical_1 write_network_cc (char_1* outfile_a);
  logical_1 init_weights_cc (int_4 rows_a, int_4 cols_a, float_4** weight_a, 
			     char_1* file_a);
  logical_1 print_weights_cc (int_4 rows_a, int_4 cols_a, float_4** weight_a, 
			      char_1* file_a);

  // load methods
  // 
  logical_1 load_spell_cc (char_1* name_a);        // load input buffer
  logical_1 read_pron_cc (FILE* file_a, int_4 length_a, 
			  char_1** pron_a);   // read pronunciation from file
  logical_1 shift_word_cc ();   // shift input buffer
  logical_1 input_empty_cc ();  // check if input buffer is empty
  
  // compute methods
  //
  logical_1 tgt_out_cc (char_1* phone_a, float_4* outs_a);  // the target bits
  logical_1 total_error_cc ();                     // total output error
  logical_1 compute_outputs_cc ();                 // outputs of neuron layers
  logical_1 compute_errors_cc (float_4* out_a);    // errors in outputs
  logical_1 compute_updates_cc (float_4* out_a);   // weight update matrices
  logical_1 update_weight_cc (float_4** up_weight_a, float_4* error_a,
			      float_4* inputs_a, int_4 row_a, int_4 col_a);
  
  // update methods
  //
  logical_1 up_momentum_cc ();      // update momentum
  logical_1 up_temperature_cc ();   // update temperature
  logical_1 up_weights_cc ();	    // update weight matrices
  logical_1 add_matrix_cc (float_4** mat1_a, float_4** mat2_a, int_4 rows_a, 
			   int_4 cols_a);   	   // add two matrices A += B
};

// end of file
//
#endif;
