// file: $ISIP_PRON/src/decision_tree/v1.0/util/generate_tree/generate_tree.cc
//

// system include files
//
#include <stdio.h>

// isip include files
//
#include <tree.h>
#include "generate_tree.h"

#include <integral_constants.h>
#include <tree_constants.h>
#include "generate_tree_constants.h"

// main program
//
main(int_4 argc, char_1** argv) {

  // declare command line arguments and set default value for
  // mode (train), context length (3), debug level (no debug)
  //
  int_4 mode = GTREE_DEFAULT_MODE;
  int_4 context = GTREE_DEFAULT_CONTEXT;
  int_4 nbest = GTREE_DEFAULT_NBEST;
  int_4 debug_level = GTREE_DEFAULT_DEBUG;  
  
  char_1 in_fname[ISIP_MAX_FNAME_SIZE];
  char_1 out_fname[ISIP_MAX_FNAME_SIZE];
  char_1 tree_fname[ISIP_MAX_FNAME_SIZE];
  
  // declare objects with fatal error mode
  //
  Tree root(ISIP_TRUE);

  // parse the command line
  //
  if (get_parameters_cc(mode, context, nbest, in_fname, out_fname, tree_fname,
			debug_level, argc, argv) != ISIP_TRUE) {
    fprintf(stdout, "<%s: error in command line arguments>\n", argv[0]);
    return ISIP_ERROR;
  }

  // set the internal debug mode
  //
  if (root.set_debug_cc(debug_level) != ISIP_TRUE) {
    fprintf(stdout, "<%s: invalid debug level>\n", argv[0]);
    return ISIP_ERROR;
  }    
  
  // echo command line options if debug is on
  //
  if (debug_level > ISIP_DEBUG_NONE) {
    fprintf(stdout, "mode: %ld\n", mode);
    fprintf(stdout, "context: %ld\n", context);
    fprintf(stdout, "input file: %s\n", in_fname);
    fprintf(stdout, "output file: %s\n", out_fname);
    fprintf(stdout, "debug level: %ld\n", debug_level);
    fprintf(stdout, "\n");
  }
  
  ////////////////////////////////////////////////////////////
  // train mode can only do one mode at a time
  //
  if (mode == GTREE_TRAIN_MODE) {

    // declare buffers for train data
    //
    int_4 train_num_items = 0;
    char_1** train_attributes;
    char_1** train_classes;
    int_4 train_array_size = (int_4)0;
    
    // load train data
    //
    if (debug_level > ISIP_DEBUG_DETAILED) {
      fprintf(stdout, "Training data....\n");
    }

    // load the training data
    //
    if (load_train_cc(train_attributes, train_classes, train_num_items,
		      train_array_size, in_fname, debug_level) != ISIP_TRUE) {
      fprintf(stdout, "%s: error loading data\n", argv[0]);
      return ISIP_ERROR;
    }

    // train the tree
    //
    if (debug_level > ISIP_DEBUG_NONE) {
      fprintf(stdout, "\nBegin training tree...\n");
    }

    if (root.train_cc(((Tree*)NULL), train_attributes, train_classes,
		      train_num_items, context) != ISIP_TRUE) {
      fprintf(stdout, "unable to train tree\n");
      return ISIP_ERROR;
    }
    
    // print the tree out to a file
    //
    FILE* fp_out = fopen((char*)out_fname, "w");
    if (fp_out == (FILE*)NULL) {
      fprintf(stdout, "unable to create file %s for writing\n", out_fname);
      return ISIP_ERROR;
    }  
    
    if (debug_level > ISIP_DEBUG_NONE) {
      fprintf(stdout, "\nPrinting tree...\n");
    }
    
    if (root.write_cc(fp_out) != ISIP_TRUE) {
      fprintf(stdout, "unable to print tree\n");
      return ISIP_ERROR;
    }
    
    fclose(fp_out);

    // free memory
    //
    for (int i=0; i<train_num_items; i++) {
      if (train_attributes[i] != (char_1*)NULL) {
	delete [] train_attributes[i];
      }
      if (train_classes[i] != (char_1*)NULL) {
	delete [] train_classes[i];
      }
    }
    if (train_attributes != (char_1**)NULL) {
      delete [] train_attributes;
    }
    
    if (train_classes != (char_1**)NULL) {
      delete [] train_classes;
    }
  }
  
  ////////////////////////////////////////////////////////////
  // test mode
  //
  else if (mode == GTREE_TEST_MODE) {

    // read in the tree
    //
    FILE* fp_in = fopen((char*)tree_fname, "r");
    if (fp_in == (FILE*)NULL) {
      fprintf(stdout, "unable to open file %s for reading\n", tree_fname);
      return ISIP_ERROR;
    }
    
    if (root.read_cc(fp_in) != ISIP_TRUE) {
      fprintf(stdout, "unable to read in the tree\n");
      return ISIP_ERROR;
    }

    fclose(fp_in);
    
    // test the tree
    //
    if (debug_level > ISIP_DEBUG_DETAILED) {
      fprintf(stdout, "Test tree....\n");
    }

    // output file
    //
    FILE* fp_out = fopen((char*)out_fname, "w");
    if (fp_out == (FILE*)NULL) {
      fprintf(stdout, "unable to open file for writing\n");
      return ISIP_ERROR;
    }

    // input file containing test data
    //
    fp_in = fopen((char*)in_fname, "r");
    if (fp_in == (FILE*)NULL) {
      fprintf(stdout, "unable to open file for reading\n");
      return ISIP_ERROR;
    }

    // misc variables
    //
    char_1 attr[ISIP_MAX_STRING_LENGTH];
    char_1 cl[ISIP_MAX_STRING_LENGTH];
    float_4 prob = 1;

    // read test data one by one
    //
    while (fgets((char*)attr, ISIP_MAX_STRING_LENGTH, fp_in)
	   != (char*)NULL) {

      attr[strlen((char*)attr)-1] = ISIP_NULL;

      // test each data string n_best times
      //
      if (root.test_cc(fp_out, nbest, attr, cl, prob) != ISIP_TRUE) {
	fprintf(stdout, "unable to test data\n");
	return ISIP_ERROR;
      }
    }
    fclose(fp_in);
    fclose(fp_out);
  }

  // this mode simply tests the I/O by reading and writing a tree
  //
  else if (mode == GTREE_TEST_IO_MODE) {  
    
    // read in the tree
    //
    FILE* fp_in = fopen((char*)in_fname, "r");
    if (fp_in == (FILE*)NULL) {
      fprintf(stdout, "unable to open file %s for reading\n", in_fname);
      return ISIP_ERROR;
    }
    
    if (root.read_cc(fp_in) != ISIP_TRUE) {
      fprintf(stdout, "unable to read in the tree\n");
      return ISIP_ERROR;
    }
    
    fclose(fp_in);
    
    // print the tree out to a file
    //
    FILE* fp_out = fopen((char*)out_fname, "w");
    if (fp_out == (FILE*)NULL) {
      fprintf(stdout, "unable to create file %s for writing\n", out_fname);
      return ISIP_ERROR;
    }  
    
    if (debug_level > ISIP_DEBUG_NONE) {
      fprintf(stdout, "\nPrinting tree...\n");
    }
    
    if (root.write_cc(fp_out) != ISIP_TRUE) {
      fprintf(stdout, "unable to print tree\n");
      return ISIP_ERROR;
    }
    
    fclose(fp_out);
  }

  ////////////////////////////////////////////////////////////
  // bad mode
  //
  else {
    fprintf(stdout, "bad mode\n");
    return ISIP_ERROR;
  }
  
  // exit gracefully
  //
  return ISIP_NO_ERROR;
}

