// file: map_phones.c
//
// functions that perform the phone mapping from Arpabet (used in rsynth) and
// Worldbet (used by the pronunciation generation software)
//

// local include files
//
#include "text_to_speech.h"
#include "text_to_speech_constants.h"

// contain arpabet and worldbet phone tables
//
char** arpabet_phones = (char**)NULL;
char** worldbet_phones = (char**)NULL;

// contain index array that tells the order of arpabet phones
//
int* index = (int*)NULL;

// contain number of items in phone table
//
int num_items = 0;

// contain the array size of phone table
//
int array_size = 0;

//////////////////////////////////////////////////////////////////////////
//
// function: get_parameters_c
//
// arguments:
//   char* s_a: (output) contain word to be transcribed into pronunciation
//   char* map_file_a: (output) contain name of transcription file
//   char* output_file_a: (output) contain name of output text file
//   int* pipe_flag__a: (output) contain mode of input
//   int* audio_flag_a: (output) contain mode of output
//   int* word_flag_a: (output) indicate if phone is entered
//   int* phone_flag_a: (output) indicate if word is entered
//   float* sf_a: (output) indicate sampling frequency
//   float* speed_a: (output) indicate play speed
//   int argc_a: (input) contain the number of arguments in the command line
//   char** argv_a: (input) contain the command line arguments
//
// return:
//   an int to indicate status
//
// This function parses the parameters from the command line arguments and
// sets appropriate flags and buffers.
//
//
int get_parameters_c(s_a, map_file_a, output_file_a, pipe_flag_a, 
		     audio_flag_a, word_flag_a, phone_flag_a, sf_a, speed_a,
		     argc_a, argv_a) 
     char* s_a;
     char* map_file_a;
     char* output_file_a;
     int* pipe_flag_a;
     int* audio_flag_a;
     int* word_flag_a;
     int* phone_flag_a;
     float* sf_a;
     float* speed_a;
     int argc_a;
     char** argv_a;
{
  // last index in command line arguments
  //
  int last_index;
  
  // local flag to indicate if -pipe option is given (default is not given). 
  // if not given, it's assumed that the argument immediately followed -word
  // or -phone option is the argument for that option
  //
  int stdin_mode = FALSE;
  
  // temporary string for miscellaneous processing
  //
  char tmp_string[MAX_STRING_LENGTH];
  
  // loop index
  //
  int index, i;
  
  // main decoding loop
  //
  last_index = argc_a;
  
  // if no arguments are given then print out a help message
  //
  if (last_index == 1) {
    sprintf(tmp_string, TTS_HELP_FORMAT, TTS_HELP_FILE);
    system(tmp_string);
    
    // exit gracefully
    //
    exit (NO_ERROR);
  } 
  
  for (index = 1; index < last_index; index++) {
    
    // check for help
    //
    if (strcmp(TTS_HELP_OPT, argv_a[index]) == 0) {
      sprintf(tmp_string, TTS_HELP_FORMAT, TTS_HELP_FILE);
      system(tmp_string);
      
      // exit gracefully
      //
      exit (NO_ERROR);
    }
    
    // pick off map file option
    //
    else if (strcmp(TTS_MAP_OPT, argv_a[index]) == 0) {
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no map file is given\n");
	return FALSE;
      }
      strcpy(map_file_a, argv_a[++index]);
    }
    
    // pick off input mode option
    //
    else if (strcmp(TTS_PIPE_OPT, argv_a[index]) == 0) {
      *pipe_flag_a = TRUE;
    }
    
    // pick off audio option
    //
    else if (strcmp(TTS_AUDIO_OPT, argv_a[index]) == 0) {
      *audio_flag_a = TRUE;
    }
    
    // pick off output file option
    //
    else if (strcmp(TTS_OUTPUT_OPT, argv_a[index]) == 0) {
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no output file is given\n");
	return FALSE;
      }
      strcpy(output_file_a, argv_a[++index]);
    }
    
    // pick off word option
    //
    else if (strcmp(TTS_WORD_OPT, argv_a[index]) == 0) {
      
      // find out if we have a -pipe option. if so, we don't have argument
      // for -word.
      //
      for (i = 1; i < last_index; i++) {
	if (strcmp(TTS_PIPE_OPT, argv_a[i]) == 0) {
          stdin_mode = TRUE;
	  break;
	}
      }
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no word is given\n");
	return FALSE;
      }
      
      if (stdin_mode != TRUE) {
	strcpy(s_a, argv_a[++index]);
      }
      *word_flag_a = TRUE;
      stdin_mode = FALSE;
    }
    
    // pick off phone option
    //
    else if (strcmp(TTS_PHONE_OPT, argv_a[index]) == 0) {
      
      // find out if we have a pipe option. if so, we don't have argument
      // for -phone
      //
      for (i = 1; i < last_index; i++) {
	if (strcmp(TTS_PIPE_OPT, argv_a[i]) == 0) {
	  stdin_mode = TRUE;
	  break;
	}
      }
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no phone is given\n");
	return FALSE;
      }
      
      if (stdin_mode != TRUE) {
	strcpy(s_a, argv_a[++index]);
      }
      *phone_flag_a = TRUE;
      stdin_mode = FALSE;
    }
    
    // pick off sampling frequency
    //
    else if (strcmp(TTS_SF_OPT, argv_a[index]) == 0) {
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no sampling frequency is given\n");
	return FALSE;
      }
      *sf_a = atof(argv_a[++index]);
    }
    
    // pick off play speed
    //
    else if (strcmp(TTS_SPEED_OPT, argv_a[index]) == 0) {
      
      // make sure that argument to option is given
      //
      if (last_index-1 == index) {
	fprintf(stdout, "get_parameters_c: no speed is given\n");
	return FALSE;
      }
      *speed_a = atof(argv_a[++index]);
    }
  }
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
//
// function: compare_str_c
//
// arguments:
//   char* str1_a: (input) contain string to be compared
//   char* str2_a: (input) contain string to be compared
//
// return:
//   an int to tell what order
//
// This function compares two strings according to their lengths, not
// alphabetical order.
//
//
int compare_str_c(str1_a, str2_a)
  char* str1_a;
char* str2_a;
{
  // compare in descending order 3 2 1
  //
  if (strlen(*(char**)str1_a) < strlen(*(char**)str2_a)) {
    return 1;
  }
  
  if (strlen(*(char**)str1_a) > strlen(*(char**)str2_a)) {
    return -1;
  }
  
  return 0;
}

//////////////////////////////////////////////////////////////////////////
//
// function: qsort_c
//
// arguments:
//   int* index_a: (output) index array to tell the order of phones
//   char** values_a: (input) contain table to sort
//   int num_items_a: (input) number of items in table to be sorted
//
// return:
//   an int to indicate status
//
// This function sorts a list of strings.  This program doesn't change
// the order of the strings in the buffer but outputs an index arrays
// that tells the order of the strings.
//
//
int qsort_c(index_a, values_a,  num_items_a) 
  int* index_a;
char** values_a;
int num_items_a;
{
  
  // copy the data into a temporary space
  // can't do a memcpy here because space is not contiguous
  //
  char* temp[num_items_a];
  int i, j;
  
  for (i = 0; i < num_items_a; i ++) {
    temp[i] = (char*)malloc(sizeof(char)*(strlen(values_a[i]) + 1));
    strcpy(temp[i], values_a[i]);
  }
  
  // sort the temp array
  //
  qsort((char**)temp, (int)num_items_a, sizeof(char*), compare_str_c);
  
  // compute an index array using a replacement algorithm
  //
  for (i = 0; i < num_items_a; i++) {
    for (j = 0; j < num_items_a; j++) {
      if (strcmp(temp[i], values_a[j]) == 0) {
	index_a[i] = j;
	free (temp[i]);
	temp[i] = (char*)NULL;
	break;
      }
    }
  }
  
  // cleanup temp variables
  //
  free (temp);
  
  // exit gracefully
  //
  return TRUE;
}


//////////////////////////////////////////////////////////////////////////
//
// function: grow_arrays_c
//
// arguments:
//   char*** ptr_strings_a: (input/output) the array to be grown
//   int* cur_size_a: (input/output) current size of array
//   int grow_size_a: (input) size to grow the array
//
// return:
//   an int to indicate status
//
// This function grows a string array given a grow size
//
//
int grow_arrays_c(ptr_strings_a, cur_size_a, grow_size_a)
  char*** ptr_strings_a;
int* cur_size_a;
int grow_size_a;
{
  
  char** new_array;
  int counter;
  
  // allocate new, larger, buffers
  //
  new_array = (char**)malloc(sizeof(char*) * (*cur_size_a + grow_size_a));
  
  // copy pointers
  //
  for (counter = 0 ; counter < *cur_size_a; counter++) {
    new_array[counter] = (*ptr_strings_a)[counter];
  }
  
  // delete the old space
  //
  free (*ptr_strings_a);
  
  // increment the size
  //
  (*cur_size_a) += grow_size_a;
  
  // swap pointers
  //
  *ptr_strings_a = new_array;
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
//
// function: map_phones_init_c
//
// arguments:
//   char* map_file_a: (input) name of transcription file
//
// return:
//   an int to indicate status
//
// This function reads the map file into the buffers for later processing.
// This map file contains the transcriptions of two pronunciation conventions.
//
// The format of the file follows this format:
//                       # comments whatever
//                       pron<tabs>pron
//                       pron<tabs>pron
//
//
int map_phones_init_c(map_file_a)
  char* map_file_a;
{
  
  // file pointer for transcription file
  //
  FILE* fp = (FILE*)NULL;
  
  // temporary variables for processings
  //
  char tmp_string[MAX_STRING_LENGTH];
  char nonwhitespace[2];  
  char map_file_exp[MAX_FNAME_LENGTH];
  
  int count = 0;
  char tmp_arpabet[MAX_STRING_LENGTH];
  char tmp_worldbet[MAX_STRING_LENGTH];
  int tmp_size;
  int i;
  
  // expand filename before open
  //
  if (expand_c(map_file_exp, map_file_a) == FALSE) {
    fprintf(stdout, "map_phones_init: unable to expand file\n");
    return FALSE;
  }
  
  fp = fopen(map_file_exp, TTS_READ);
  if (fp == (FILE*)NULL) {
    return FALSE;
  }
  
  // allocate initial space
  //
  num_items = 0;
  array_size = TTS_ARRAY_SIZE;
  arpabet_phones = (char**)malloc(sizeof(char*) * array_size);
  worldbet_phones = (char**)malloc(sizeof(char*) * array_size);
  
  // read the data into the buffer
  //
  while (fscanf(fp, TTS_READ_FORMAT_A, tmp_string) != EOF) {
    
    // check for a comment line and/or blank line
    //
    if ((tmp_string[0] != TTS_COMMENT_CHAR) &&
	(tmp_string[0] != TTS_BLANK_CHAR)) {
      
      // make sure we have  enough space for new data
      //
      if(num_items >= array_size) {
	tmp_size = array_size;
	grow_arrays_c(&arpabet_phones, &tmp_size, TTS_ARRAY_SIZE);
	grow_arrays_c(&worldbet_phones, &array_size, TTS_ARRAY_SIZE);
      }
      
      // add the new data
      //
      arpabet_phones[num_items] = (char*)malloc(sizeof(char) * 
						TTS_MAX_PHONE_LENGTH);
      worldbet_phones[num_items] = (char*)malloc(sizeof(char) * 
						 TTS_MAX_PHONE_LENGTH);
      strcpy(arpabet_phones[num_items], tmp_string);
      fscanf(fp, TTS_READ_FORMAT_B, tmp_string);
      fscanf(fp, TTS_READ_FORMAT_C, worldbet_phones[num_items]);
      num_items++;
    }
    fscanf(fp, TTS_READ_FORMAT_C, tmp_string);
    fscanf(fp, TTS_READ_FORMAT_D, tmp_string);
  }
  
  // sort the map file
  //
  index = (int*)malloc(sizeof(int) * num_items);
  if (qsort_c(index, arpabet_phones, num_items) == 0) {
    fprintf(stdout, "map_phone_init_c: error while sorting\n");
    return FALSE;
  }
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
//
// function: map_phones_c
//
// arguments:
//   char* out_a: (output) phonemes after translation
//   char* in_a: (input) phonemes before translation
//   int xlate_flag_a: (input) tell what direction to translate
//
// return:
//   an int to indicate status
//
// This function maps one phonetic convention to another phonetic convention
// given an input flag to tell which phonetic convention to convert the input
// phonetics.  
//
//
int map_phones_c(out_a, in_a, xlate_flag_a)
  char* out_a;
char* in_a;
int xlate_flag_a;
{
  
  // temporary variables for miscellaneous processings
  //
  char tmp_string[MAX_STRING_LENGTH];
  int counter = 0;
  int i;
  int find_flag = FALSE;
  int token_counter = 0;
  
  // contain the phones in an input word
  //
  char* tokens[MAX_STRING_LENGTH];
  
  // contain the number of phones for a given input word
  int num_tokens;
  
  // copy arpabet into a temp buffer
  //
  strcpy(tmp_string, in_a);
  
  // parse this data into tokens using " " (blank) as delimiter
  //
  tokens[0] = strtok(tmp_string, TTS_DELIMITER);
  num_tokens = 1; 
  while (TRUE) {
    tokens[num_tokens] = strtok((char*)NULL, TTS_DELIMITER);
    if (tokens[num_tokens] ==  (char*)NULL) {
      break;
    }
    num_tokens++;
  }
  
  // because we use strcat need to reinitialize out_a
  // each time we go into this routine
  //
  out_a[0] = NULL_CHAR;
  
  // find a match for all the tokens
  //
  for (i = 0; i < num_tokens; i++) {
    
    // go to the beginning of the table
    //
    counter = 0;
    
    if (xlate_flag_a == TTS_WORLDBET) {
      
      while (counter < num_items) {
	
	// compare if phone match
	//
	if (strcmp(tokens[i], arpabet_phones[index[counter]]) == 0) {
	  
	  // if match, get equivalent worldbet phone
	  //
	  strcat(out_a, worldbet_phones[index[counter]]);
	  
	  // put delimiter between each phone
	  //
	  strcat(out_a, TTS_DELIMITER);
	  
	  // increment counter
	  //
          token_counter++;
	  
	  break;
	}
	counter++;
      }
    }
    else if (xlate_flag_a == TTS_ARPABET) {
      
      while (counter < num_items) {
	
	// compare if phone match
	//
	if (strcmp(tokens[i], worldbet_phones[counter]) == 0) {
	  
	  // if match, get equivalent worldbet phone
	  // do not put delimiter between each phone since original rsynth
	  // doesn't.  delimiter between each phone will affect the audio
	  //
	  strcat(out_a, arpabet_phones[counter]);
          
          // increment counter
          //
          token_counter++;
	  break;
	}
	counter++;
      }
    }
  }
  
  if (token_counter != num_tokens) {
    fprintf(stdout, "map_phones_c: match was not found for given phone\n");
    return FALSE;
  }
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
//
// function: map_phones_free_c
//
// arguments:
//   none
//
// return:
//   an int to indicate status
//
// This function releases the space allocated to buffers that are used
// to store the phonetic conversion tables.
//
//
int map_phones_free_c()
{
  int i;
  
  for (i = 0; i < num_items; i++) {
    free(arpabet_phones[i]);
    free(worldbet_phones[i]);
  }
  free(arpabet_phones);
  free(worldbet_phones);
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
//
// function: to_lower_c
//
// arguments:
//   char* s_a: (input/output) string to be converted to lower case
//
// return:
//   an int to indicate status
//
// This function converts all upper case letters to lower case letters.
// This is to make rsynth happy because rsynth will complain if it
// dectects upper case letters.
//
//
int to_lower_c(s_a)
  char* s_a;
{
  int i;
  for (i = 0; i < strlen(s_a); i++) {
    if (isupper(s_a[i])) {
      s_a[i] = tolower(s_a[i]);
    }
  }
  
  // exit gracefully
  //
  return TRUE;
}


//////////////////////////////////////////////////////////////////////////
//
// function: text_to_phones_c
//
// arguments:
//   char* worldbet_equ_a: (output) output phonemes of word
//   char* s_a: (input) string to be converted to lower case
//
// return:
//   an int to indicate status
//
// This function converts the text to phonetic symbols using rsynth routines.
//
//
int text_to_phones_c(worldbet_equ_a, s_a)
  char* worldbet_equ_a;
char* s_a;
{
  
  char arpabet_equ[MAX_STRING_LENGTH];
  
  // convert to lower case first
  //
  to_lower_c(s_a);
  
  // get rsynth translation
  //
  isip_say_string(arpabet_equ, s_a);
  
  // convert arpabet to worldbet
  //
  if (map_phones_c(worldbet_equ_a, arpabet_equ, TTS_WORLDBET) == FALSE) {
    fprintf(stdout, "text_to_phones_c: unable to transcribe from arpabet to worldbet for %s\n", s_a);
    return FALSE;
  }
  
  // exit gracefully
  //
  return TRUE;
}


//////////////////////////////////////////////////////////////////////////
//
// function: phones_to_speech_c
//
// arguments:
//   char* audio_file_a: (output) output audio file
//   char* worldbet_equ_a: (input) phones to be converted to speech
//
// return:
//   an int to indicate status
//
// This function converts phonetic symbols to audio using rsynth routines.
// The one constraint is that because we use worldbet and rsynth uses arpabet
// we have to convert our worldbet to arpabet so that rsynth can handle it.
//
//
int phones_to_speech_c(audio_file_a, worldbet_equ_a, sf_a, speed_a)
  char* audio_file_a;
char* worldbet_equ_a;
float sf_a;
float speed_a;
{
  
  int argc;
  char** argv;
  int i;
  
  char arpabet_equ[MAX_STRING_LENGTH];
  darray_t phone;
  
  // simulate command line argument for rsynth to work
  //
  argc = TTS_RSYNTH_OPTS;
  argv = (char**)malloc(sizeof(char*) * argc);
  for (i = 0; i < argc; i++) {
    argv[i] = (char*)malloc(sizeof(char) * MAX_STRING_LENGTH);
  }
  
  // use audio
  //
  strcpy(argv[0], TTS_RSYNTH_AUDIO);
  
  // output au file
  //
  strcpy(argv[1], TTS_RSYNTH_OUTPUT);
  strcpy(argv[2], audio_file_a);
  
  // sampling rate
  //
  strcpy(argv[3], TTS_RSYNTH_SF);
  sprintf(argv[4], "%f", sf_a);
  
  // play speed
  //
  strcpy(argv[5], TTS_RSYNTH_SPEED);
  sprintf(argv[6], "%f", speed_a);
  
  // initialize audio
  //
  argc = audio_init(argc, argv);
  argc = init_synth(argc, argv);
  argc = init_holmes(argc, argv);
  argc = file_init(argc, argv);
  
  // convert worldbet to arpabet
  //
  if (map_phones_c(arpabet_equ, worldbet_equ_a, TTS_ARPABET) == FALSE) {
    fprintf(stdout, "phones_to_speech_c: unable to translate from worldbet to arpabet for %s\n", worldbet_equ_a);
    return FALSE;
  }
  
  // simulate darray_t structure for rsynth to work
  //
  phone.data = (char*)malloc(sizeof(char)*(strlen(arpabet_equ)+1));
  strcpy(phone.data, arpabet_equ);
  phone.items = strlen(arpabet_equ);
  phone.alloc = 128;
  phone.esize = 1;
  phone.get = 128;
  
  // convert arpabet to speech
  //
  say_phones((char*)darray_find(&phone, 0), phone.items, 0);
  
  free(phone.data);
  
  // exit gracefully
  //
  return TRUE;
}


//////////////////////////////////////////////////////////////////////////
//
// function: process_word_c
//
// arguments:
//   char* s_a: (input) word to be transcribed
//   char* output_file_a: (input) output file name if any
//   int pipe_flag_a: (input) flag to tell if input is from stdin
//   int audio_flag_a: (input) flag to tell if output is in audio form
//
// return:
//   an int to indicate status
//
// This function processes the -word option. One limitation at the moment is
// that no audio file will be produce for multiple inputs.
//
//
int process_word_c(s_a, output_file_a, pipe_flag_a, audio_flag_a, sf_a, speed_a)
  char* s_a;
char* output_file_a;
int pipe_flag_a;
int audio_flag_a;
float sf_a;
float speed_a;
{
  // temporary variables
  //
  char tmp_file[MAX_FNAME_LENGTH];
  char cmd[MAX_STRING_LENGTH];
  
  char worldbet_equ[MAX_STRING_LENGTH];
  
  // prepare an output file if any
  //
  FILE* fp_out = (FILE*)NULL;
  
  // make a unique filename
  //
  strcpy(tmp_file, TTS_TEMPLATE);
  if (strlen(mktemp(tmp_file)) == 0) {
    fprintf(stdout, "process_word_c: unable to create temporary file\n");
    return FALSE;
  }
  
  // attach appropriate extension to different kinds of files
  //
  strcat(tmp_file, TTS_FILE_EXT);
  
  // initialize variable
  //
  memset(worldbet_equ, NULL_CHAR, MAX_STRING_LENGTH);
  
  // read from stdin
  //
  if (pipe_flag_a == TRUE) {
    
    // output to a file
    //
    if (output_file_a[0] != NULL_CHAR) {
      fp_out = fopen(output_file_a, TTS_WRITE);
      
      // audio output
      //
      if (audio_flag_a == TRUE) {
	fprintf(stdout, "process_word_c: -audio is not allowed with -pipe option\n");
	return FALSE;
      }
      
      // pronunciation output
      //
      else {
	while (fscanf(stdin, TTS_READ_FORMAT_C, s_a) != EOF) {
	  if (text_to_phones_c(worldbet_equ, s_a) != FALSE) {
	    if (fp_out != (FILE*)NULL) {
	      fprintf(fp_out, TTS_PRINT_FORMAT_A, s_a, worldbet_equ);
	    }
	  }
	  else {
	    fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  }
	  fscanf(stdin, TTS_READ_FORMAT_D, s_a);
	}
      }
    }
    
    // output to stdout if no output option is given
    //
    else {
      // audio output
      //
      if (audio_flag_a == TRUE) {
	fprintf(stdout, "process_word_c: -audio is not allowed with -pipe option\n");
	return FALSE;
      }
      
      // pronunciation output
      //
      else {
	while (fscanf(stdin, TTS_READ_FORMAT_C, s_a) != EOF) {
	  if (text_to_phones_c(worldbet_equ, s_a) != FALSE) {
	    fprintf(stdout, TTS_PRINT_FORMAT_A, s_a, worldbet_equ);
	  }
	  else {
	    fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  }
	  fscanf(stdin, TTS_READ_FORMAT_D, s_a);
	}
      }
    }
  }
  
  // read from option
  //
  else {
    
    // output to a file
    //
    if (output_file_a[0] != NULL_CHAR) {
      fp_out = fopen(output_file_a, TTS_WRITE);
      
      // audio output
      //
      if (audio_flag_a == TRUE) {
	if (text_to_phones_c(worldbet_equ, s_a) == FALSE) {
	  fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  return FALSE;
	}
	
	if (phones_to_speech_c(output_file_a, worldbet_equ, sf_a, speed_a)
	    == FALSE) {
	  fprintf(stdout, "unable to produce speech for word: %s\n", s_a);
	  return FALSE;
	}
      }
      
      // pronunciation output
      //
      else {
	if (text_to_phones_c(worldbet_equ, s_a) == FALSE) {
	  fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  return FALSE;
	}
	
	if (fp_out != (FILE*)NULL) {
	  fprintf(fp_out, TTS_PRINT_FORMAT_A, s_a, worldbet_equ);
	}
      }
    }
    
    // output to stdout if no output option is given
    //
    else {
      // audio output
      //
      if (audio_flag_a == TRUE) {
	if (text_to_phones_c(worldbet_equ, s_a) == FALSE) {
	  fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  return FALSE;
	}
	
	if (phones_to_speech_c(tmp_file, worldbet_equ, sf_a, speed_a)
	    == FALSE) {
	  fprintf(stdout, "unable to produce speech for word: %s\n", s_a);
	  return FALSE;
	}
	
	// display audio file to stdout
	//
	sprintf(cmd, "cat %s", tmp_file);
	system(cmd);
	
	// remove any temp files
	//
	sprintf(cmd, "rm %s", tmp_file);
	system(cmd);
      }
      
      // pronunciation output
      //
      else {
	if (text_to_phones_c(worldbet_equ, s_a) == FALSE) {
	  fprintf(stdout, "unable to transcribe word: %s\n", s_a);
	  return FALSE;
	}
	fprintf(stdout, TTS_PRINT_FORMAT_A, s_a, worldbet_equ);
      }
    }
  }
}



//////////////////////////////////////////////////////////////////////////
//
// function: process_phone_c
//
// arguments:
//   char* s_a: (input) phone to be transcribed
//   char* output_file_a: (input) output file name if any
//   int pipe_flag_a: (input) flag to tell if input is from stdin
//   int audio_flag_a: (input) flag to tell if output is in audio form
//
// return:
//   an int to indicate status
//
// This function processes -phone option. One limitation is that no audio
// file will be produce for multiple inputs.
//
//
int process_phone_c(s_a, output_file_a, pipe_flag_a, audio_flag_a, sf_a,
		    speed_a)
     char* s_a;
     char* output_file_a;
     int pipe_flag_a;
     int audio_flag_a;
     float sf_a;
     float speed_a;
{
  
  // temporary variables
  //
  char tmp_file[MAX_FNAME_LENGTH];
  char cmd[MAX_STRING_LENGTH];
  
  FILE* fp_out = (FILE*)NULL;
  
  // make a unique filename
  //
  strcpy(tmp_file, TTS_TEMPLATE);
  if (strlen(mktemp(tmp_file)) == 0) {
    fprintf(stdout, "process_word_c: unable to create temporary file\n");
    return FALSE;
  }
  
  // input phones might have blanks, strip them off before processing
  //
  remove_blanks_c(s_a);

  // attach appropriate extension to different kinds of files
  //
  strcat(tmp_file, TTS_FILE_EXT);
  
  // prepare an output file if any
  //
  if (output_file_a[0] != NULL_CHAR) {
    fp_out = fopen(output_file_a, TTS_WRITE);
  }
  
  // read from stdin
  //
  if (pipe_flag_a == TRUE) {
    fprintf(stdout, "process_phones_c: -pipe option is not allow with -phone option\n");
    return FALSE;
  }
  
  // read from option
  //
  else {
    
    // output to a file
    //
    if (output_file_a[0] != NULL_CHAR) {
      if (phones_to_speech_c(output_file_a, s_a, sf_a, speed_a) == FALSE) {
	fprintf(stdout, "unable to produce speech for phone: %s\n", s_a);
	return FALSE;
      }
    }
    
    // output to stdout if no output option is given
    //
    else {
      if (phones_to_speech_c(tmp_file, s_a, sf_a, speed_a) == FALSE) {
	fprintf(stdout, "unable to produce speech for word: %s\n", s_a);
	return FALSE;
      }
      
      // display audio file to stdout
      //
      sprintf(cmd, "cat %s", tmp_file);
      system(cmd);
      
      // remove any temp files
      //
      sprintf(cmd, "rm %s", tmp_file);
      system(cmd);
    }
  }
}


//////////////////////////////////////////////////////////////////////////
// 
// function: expand_c
//
// arguments:
//  char* o_name: (output) name with all environmental symbols and variables 
//                expanded
//  char* i_name: (input) name needing expansion
//
// return: 
//  an int indicating status
//
// This function replaces all environment variables and other such
// things in a string with their equivalent expansions. For example:
// 
//  $HOME/foo.text              => /u1/person/foo.text
//  foo.text/$HOME/data         => foo.text//u1/person/data
//
// in other words, variables or aliases anywhere in the string
// will be replaced.
//
// This method now uses popen/pclose rather than complicated
// function calls and string replacements.
//
int expand_c(o_name_a, i_name_a) 
  char* o_name_a;
char* i_name_a;
{
  
  // temporary variables for processing
  //
  int len;
  int flag_match;
  int i;
  char buf[MAX_FNAME_LENGTH];
  FILE* fp;
  
  // scan the incoming string for special characters
  //
  len = strlen(i_name_a);
  flag_match = FALSE;
  
  for (i=0; i<len; i++) {
    if (strchr(TTS_ENV_DELIMITERS, i_name_a[i]) != (char*)NULL) {
      flag_match = TRUE;
      break;
    }
  }
  
  // if necessary, expand the string
  //
  if (flag_match == TRUE) {
    
    // build a Unix command
    //
    sprintf(buf, "echo %s\n", i_name_a);
    
    // execute the pipe
    //
    fp = popen(buf, "r");
    fgets(o_name_a, MAX_FNAME_LENGTH, fp);
    pclose(fp);
    
    // strip the trailing linefeed
    //
    len = strlen(o_name_a) - 1;
    o_name_a[len] = NULL_CHAR;
    
    // make sure everything got expanded
    //
    flag_match = FALSE;
    for (i=0; i<len; i++) {
      if (strchr(TTS_ENV_DELIMITERS, o_name_a[i]) != (char*)NULL) {
	return FALSE;
      }
    }
  }
  
  // else, just copy the string
  //
  else {
    strcpy(o_name_a, i_name_a);
  }
  
  // exit gracefully
  //
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// 
// function: remove_blanks_c
//
// arguments:
//  char* str_a: (input/output) 
//
// return: 
//  none
////////////////////////////////////////////////////////////////////////////

// This function strips the pronunciations of blank phones.
//
void remove_blanks_c (char* str_a)
{
  int length;
  int i, j;
  
  // make sure there is a blank space at the end of the phone string
  // this is required in case the last phone is a blank
  //
  strcat(str_a, " ");
 
  // length of string
  //
  length = strlen(str_a) + 1;

  // check for blank phone symbol
  //
  for (i = 0; i < length; i++) {
    if (str_a[i] == '_') {
      length -= 2;
      for (j = i; j < length; j++) {
        str_a[j] = str_a[j + 2];
      }
      i -= 2;
    }
  }
  
  // exit function gracefully
  //
}

// end of file
//
